@@ -21,6 +21,8 @@
 #include <kconfigwatcher.h>
 #include <ksharedconfig.h>
 
+#include <private/qlocale_p.h> // QSystemLocale
+
 #ifdef Q_OS_UNIX
 #include <utime.h>
 #endif
@@ -28,6 +30,8 @@
 #include <unistd.h> // getuid
 #endif
 
+using namespace Qt::StringLiterals;
+
 KCONFIGGROUP_DECLARE_ENUM_QOBJECT(KConfigTest, Testing)
 KCONFIGGROUP_DECLARE_FLAGS_QOBJECT(KConfigTest, Flags)
 
@@ -771,12 +775,22 @@ void KConfigTest::testDeleteEntry()
     KConfig sc(configFile);
     KConfigGroup group(&sc, QStringLiteral("Hello"));
 
+    QVERIFY(!sc.isDirty());
     group.deleteEntry("DelKey");
+    QVERIFY(sc.isDirty());
     QCOMPARE(group.readEntry("DelKey", QStringLiteral("Fietsbel")), QStringLiteral("Fietsbel"));
 
     QVERIFY(group.sync());
     Q_ASSERT(!readLines(configFile).contains("DelKey=ToBeDeleted\n"));
     QCOMPARE(group.readEntry("DelKey", QStringLiteral("still deleted")), QStringLiteral("still deleted"));
+
+    // deleting a deleted key again should not mark it as dirty
+    group.deleteEntry("DelKey");
+    QVERIFY(!sc.isDirty());
+
+    // and neither should deleting an unknown key
+    group.deleteEntry("DoesNotExist");
+    QVERIFY(!sc.isDirty());
 }
 
 void KConfigTest::testDelete()
@@ -1547,6 +1561,43 @@ void KConfigTest::testLocaleConfig()
     QFile::remove(file);
 }
 
+void KConfigTest::testLocaleConfigWithOverrideLanguage()
+{
+    // simulate usage of language override (kxmlgui);
+    // we need to create a new QSystemLocale to make Qt update the locale
+    qputenv("LANGUAGE", "ca:de_DE");
+    QSystemLocale newSystemLocale{};
+
+    // Initialize the testdata
+    QDir().mkpath(m_testConfigDir);
+    const QString file = m_testConfigDir + QLatin1String("/localized.test");
+    QFile::remove(file);
+    QFile f(file);
+    QVERIFY(f.open(QIODevice::WriteOnly));
+    QTextStream ts(&f);
+    ts << "[Test]\n";
+    ts << "foo=4\n";
+    ts << "foo[ca]=5\n";
+    ts << "foostring=primary\n";
+    ts << "foostring[ca]=nice\n";
+    ts << "foobool=primary\n";
+    ts << "foobool[ca]=true\n";
+    f.close();
+
+    // Load the testdata
+    QVERIFY(QFile::exists(file));
+    KConfig config(file);
+    QVERIFY(config.locale().startsWith("ca"_L1));
+
+    KConfigGroup cg2(&config, QStringLiteral("Test"));
+    QCOMPARE(cg2.readEntry("foo", 3), 5);
+    QCOMPARE(cg2.readEntry("foostring"), QStringLiteral("nice"));
+    QCOMPARE(cg2.readEntry("foobool", false), true);
+
+    // Clean up after the testcase
+    QFile::remove(file);
+}
+
 void KConfigTest::testDeleteWhenLocalized()
 {
     // Initialize the testdata
